# pylint: disable=missing-docstring,no-self-use,redefined-builtin,too-many-arguments
# pylint: disable=protected-access

"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import json
import os
import unittest
import argparse
import mock

from icetea_lib.TestSuite.TestcaseContainer import TestcaseContainer, TestStatus
from icetea_lib.Result import Result
from icetea_lib.bench import Bench


def mock_load_tc(*args, **kwargs):  # pylint: disable=unused-argument
    return None


def mock_raise_type(*args, **kwargs):  # pylint: disable=unused-argument
    raise TypeError()


def mock_raise_import(*args, **kwargs):  # pylint: disable=unused-argument
    raise ImportError()


class MockInstance(object):
    def __init__(self, name, version, type=None, skip_val=True, skip_info=None):
        self.info = skip_info if skip_info else {"only_type": "process"}
        self.config = {
            "compatible": {
                "framework": {
                    "name": name, "version": version}
            },
            "requirements": {
                "duts": {
                    "*": {
                        "type": type
                    }
                }
            }
        }
        self.skip_val = skip_val

    def get_result(self):
        return Result()

    def get_test_name(self):
        return "Icetea"

    def skip(self):
        return self.skip_val

    def skip_info(self):
        return self.info

    def skip_reason(self):
        return "test"


class TCContainerTestcase(unittest.TestCase):

    def setUp(self):
        with open(os.path.join("./icetea_lib", 'tc_schema.json')) as data_file:
            self.tc_meta_schema = json.load(data_file)

        self.args_tc = argparse.Namespace(
            available=False, version=False, bin=None, binary=False, channel=None,
            clean=False, cloud=False, component=False, device='*', gdb=None,
            gdbs=None, gdbs_port=2345, group=False, iface=None, kill_putty=False, list=False,
            listsuites=False, log='./log', my_duts=None, nobuf=None,
            pause_when_external_dut=False, putty=False, reset=False, silent=True,
            skip_case=False, skip_rampdown=False, skip_rampup=False,
            status=False, suite=None, tc="test_cmdline", tc_cfg=None, tcdir="examples",
            testtype=False, type="process", subtype=None, use_sniffer=False,
            valgrind=False, valgrind_tool=None, verbose=False, repeat=0, feature=None,
            suitedir="./test/suites", forceflash_once=True, forceflash=True,
            stop_on_failure=False, ignore_invalid_params=False)

    @mock.patch("icetea_lib.TestSuite.TestcaseContainer.load_class")
    def test_load_testcase_fails(self, mock_loadclass):
        testcase = TestcaseContainer.find_testcases("examples.test_cmdline", "./examples",
                                                    self.tc_meta_schema)[0]
        with self.assertRaises(TypeError):
            testcase._load_testcase(1)
        mock_loadclass.side_effect = [ValueError, None]
        with self.assertRaises(ImportError):
            testcase._load_testcase("test_case")
        with self.assertRaises(ImportError):
            testcase._load_testcase("test_case")

    def test_check_major_version(self):
        testcase = TestcaseContainer.find_testcases("examples.test_cmdline", "examples",
                                                    self.tc_meta_schema)[0]
        self.assertFalse(testcase._check_major_version("1.0.0", "0.9.1"))
        self.assertFalse(testcase._check_major_version("1.0.0", ">0.0.2"))
        self.assertFalse(testcase._check_major_version("1.0.0", ">=0.0.3"))

    @mock.patch("icetea_lib.TestSuite.TestcaseContainer.get_fw_version")
    def test_version_checker(self, mock_fwver):
        mock_fwver.return_value = "0.9.0"
        testcase = TestcaseContainer.find_testcases("examples.test_cmdline", "examples",
                                                    self.tc_meta_schema)[0]
        self.assertIsNone(testcase._check_version(MockInstance("Icetea", "0.9.0")))
        res = testcase._check_version(MockInstance("Icetea", "0.2.2"))
        self.assertEqual(res.get_verdict(), "skip")
        mock_fwver.return_value = "0.2.2"
        self.assertIsNone(testcase._check_version(MockInstance("Icetea", "<0.9.0")))
        res = testcase._check_version(MockInstance("Icetea", ">0.9.0"))
        self.assertEqual(res.get_verdict(), "skip")
        mock_fwver.return_value = "0.9.0"
        self.assertIsNone(testcase._check_version(MockInstance("Icetea", ">=0.9.0")))
        mock_fwver.return_value = "0.9.1"
        self.assertIsNone(testcase._check_version(MockInstance("Icetea", ">=0.9.0")))

    def test_check_skip(self):
        testcase = TestcaseContainer.find_testcases("examples.test_cmdline", "./examples",
                                                    self.tc_meta_schema)[0]
        res = testcase._check_skip(MockInstance("test", "0.9.0", "process"))
        self.assertFalse(res)
        self.assertFalse(testcase._check_skip(MockInstance("test", "0.9.0", "hardware",
                                                           skip_val=False)))

        res = testcase._check_skip(MockInstance("test", "0.9.0", "process", True, {"test": "test"}))
        self.assertEqual(res.get_verdict(), "skip")

    def test_find_testcases(self):
        lst = TestcaseContainer.find_testcases("test.testbase.dummy_multiples", "./test/testbase",
                                               self.tc_meta_schema)
        self.assertEqual(len(lst), 2)
        lst = TestcaseContainer.find_testcases("test.testbase.dummy", "./test/testbase",
                                               self.tc_meta_schema)
        self.assertEqual(len(lst), 1)
        with self.assertRaises(TypeError):
            TestcaseContainer.find_testcases(1, "./test/testbase", self.tc_meta_schema)
        with self.assertRaises(ValueError):
            TestcaseContainer.find_testcases("", "./test/testbase", self.tc_meta_schema)

    def test_create_new_bench_instance(self):
        lst = TestcaseContainer.find_testcases("test.testbase.dummy", "./test/testbase",
                                               self.tc_meta_schema)
        inst = lst[0]._create_new_bench_instance("test.testbase.dummy")
        self.assertTrue(isinstance(inst, Bench))

    @mock.patch("icetea_lib.TestSuite.TestcaseContainer.TestcaseContainer.get_instance")
    @mock.patch("icetea_lib.TestSuite.TestcaseContainer.TestcaseContainer._check_version")
    @mock.patch("icetea_lib.TestSuite.TestcaseContainer.TestcaseContainer._check_skip")
    @mock.patch("icetea_lib.TestSuite.TestcaseContainer.get_tc_arguments")
    def test_run(self, mock_parser, mock_skip, mock_version, mock_instance):
        testcase = TestcaseContainer.find_testcases("examples.test_cmdline", "./examples",
                                                    self.tc_meta_schema)[0]
        # Initialize mocks
        parser = mock.MagicMock()
        instance = mock.MagicMock()
        instance.run = mock.MagicMock()
        instance.run.return_value = 0
        instance.get_result = mock.MagicMock()
        instance.get_result.return_value = Result()
        mock_instance.return_value = instance
        mock_skip.return_value = None
        mock_version.return_value = None
        parser.parse_known_args = mock.MagicMock()
        parser.parse_known_args.return_value = (mock.MagicMock(), [])
        mock_parser.return_value = parser
        # Mocked a succesful run
        testcase.run()

        # Skip returns 1, tc should be skipped
        mock_skip.return_value = 1
        mock_version.return_value = None
        self.assertEqual(testcase.status, TestStatus.FINISHED)

        testcase.run()

        # Version mismatch
        mock_skip.return_value = None
        mock_version.return_value = 1
        self.assertEqual(testcase.status, TestStatus.FINISHED)

        testcase.run()

        # Unknown arguments
        mock_version.return_value = None
        parser.parse_known_args.return_value = (self.args_tc, [1])

        res = testcase.run()
        self.assertEqual(testcase.status, TestStatus.FINISHED)
        self.assertEqual(res.get_verdict(), 'inconclusive')

        result = Result()
        result.retcode = 1012
        instance.get_result.return_value = result
        instance.run.return_value = 1012
        parser.parse_known_args.return_value = (mock.MagicMock(), [])
        testcase.run()


if __name__ == '__main__':
    unittest.main()



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from icetea_lib.TestStepError import TestStepFail
import inspect
import os
import json

def format_message(msg):
    """
    Formatting function for assert messages. Fetches the filename, function and line number of the code causing the fail
    and formats it into a three-line error message. Stack inspection is used to get the information.
    Originally done by BLE-team for their testcases.

    :param msg: Message to be printed along with the information.
    :return: Formatted message as string.
    """
    callerframerecord = inspect.stack()[2]
    frame = callerframerecord[0]
    info = inspect.getframeinfo(frame)
    _, filename = os.path.split(info.filename)
    caller_site = "In file {!s}, in function {!s}, at line {:d}".format(filename, info.function, info.lineno)
    return "{!s}\n{!s}\n{!s}".format(msg, caller_site, info.code_context)


def assertTraceDoesNotContain(response, message):
    """
    Raise TestStepFail if response.verify_trace finds message from response traces.

    :param response: Response. Must contain method verify_trace
    :param message: Message to look for
    :return: Nothing
    :raises: AttributeError if response does not contain verify_trace method.
    TestStepFail if verify_trace returns True.
    """
    if not hasattr(response, "verify_trace"):
        raise AttributeError("Response object does not contain verify_trace method!")
    if response.verify_trace(message, False):
        raise TestStepFail('Assert: Message(s) "%s" in response' % message)


def assertTraceContains(response, message):
    """
    Raise TestStepFail if response.verify_trace does not find message from response traces.

    :param response: Response. Must contain method verify_trace
    :param message: Message to look for
    :return: Nothing
    :raises: AttributeError if response does not contain verify_trace method.
    TestStepFail if verify_trace returns False.
    """
    if not hasattr(response, "verify_trace"):
        raise AttributeError("Response object does not contain verify_trace method!")
    if not response.verify_trace(message, False):
        raise TestStepFail('Assert: Message(s) "%s" not in response' % message)


def assertDutTraceDoesNotContain(dut, message, bench):
    """
    Raise TestStepFail if bench.verify_trace does not find message from dut traces.

    :param dut: Dut object.
    :param message: Message to look for.
    :param: Bench, must contain verify_trace method.
    :raises: AttributeError if bench does not contain verify_trace method.
    TestStepFail if verify_trace returns True.
    """
    if not hasattr(bench, "verify_trace"):
        raise AttributeError("Bench object does not contain verify_trace method!")
    if bench.verify_trace(dut, message, False):
        raise TestStepFail('Assert: Message(s) "%s" in response' % message)


def assertDutTraceContains(dut, message, bench):
    """
    Raise TestStepFail if bench.verify_trace finds message from dut traces.

    :param dut: Dut object.
    :param message: Message to look for.
    :param: Bench, must contain verify_trace method.
    :raises: AttributeError if bench does not contain verify_trace method.
    TestStepFail if verify_trace returns True.
    """
    if not hasattr(bench, "verify_trace"):
        raise AttributeError("Bench object does not contain verify_trace method!")
    if not bench.verify_trace(dut, message, False):
        raise TestStepFail('Assert: Message(s) "%s" not in response' % message)


def assertTrue(expr, message=None):
    """
    Assert that expr evaluates to True.

    :param expr: expression to evaluate.
    :param message: Message set to raised Exception
    :raises: TestStepFail if not expr.
    """
    if not expr:
        raise TestStepFail(format_message(message) if message is not None else 'Assert: %r != True' % expr)


def assertFalse(expr, message=None):
    """
    Assert that expr evaluates to False.

    :param expr: expression to evaluate.
    :param message: Message set to raised Exception
    :raises: TestStepFail if expr.
    """
    if expr:
        raise TestStepFail(format_message(message) if message is not None else 'Assert: %r != False' % expr)


def assertNone(expr, message=None):
    """
    Assert that expr is None.

    :param expr: expression.
    :param message: Message set to raised Exception
    :raises: TestStepFail if expr is not None.
    """
    if expr is not None:
        raise TestStepFail(format_message(message) if message is not None else "Assert: %s != None" % str(expr))


def assertNotNone(expr, message=None):
    """
    Assert that expr is not None.

    :param expr: expression.
    :param message: Message set to raised Exception
    :raises: TestStepFail if expr is None.
    """
    if expr is None:
        raise TestStepFail(format_message(message) if message is not None else "Assert: %s == None" % str(expr))


def assertEqual(first, second, message=None):
    """
    Assert that first equals second.

    :param first: First part to evaluate
    :param second: Second part to evaluate
    :param message: Failure message
    :raises: TestStepFail if not first == second
    """
    if not first == second:
        raise TestStepFail(format_message(message) if message is not None else "Assert: %s != %s" % (str(first), str(second)))


def assertNotEqual(first, second, message=None):
    """
    Assert that first does not equal second.

    :param first: First part to evaluate
    :param second: Second part to evaluate
    :param message: Failure message
    :raises: TestStepFail if not first != second
    """
    if not first != second:
        raise TestStepFail(format_message(message) if message is not None else "Assert: %s == %s" % (str(first), str(second)))


def assertJsonContains(jsonStr=None, key=None, message=None):
    """
    Assert that jsonStr contains key.

    :param jsonStr: Json as string
    :param key: Key to look for
    :param message: Failure message
    :raises: TestStepFail if key is not in jsonStr or
    if loading jsonStr to a dictionary fails or if jsonStr is None.
    """
    if jsonStr is not None:
        try:
            data = json.loads(jsonStr)
            if key not in data:
                raise TestStepFail(format_message(message) if message is not None else "Assert: Key : %s is not in : %s" % (str(key), str(jsonStr)))
        except (TypeError, ValueError) as e:
            raise TestStepFail(format_message(message) if message is not None else "Unable to parse json "+str(e))
    else:
        raise TestStepFail(format_message(message) if message is not None else "Json string is empty")



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import os
from ast import literal_eval as le

class TestcaseFilter(object):
    """
    TestcaseFilter class. Provides the handling for different filtering arguments.
    Provides a match function to match testcases with filter.
    """
    def __init__(self):
        self._filter = {'list': False, 'name': False, 'status': False,
                        'group': False, 'type': False, 'subtype': False,
                        'comp': False, 'feature': False, 'platform': False}

    def tc(self, value):
        # tc can be:
        # int, tuple, list or str(any of the above)
        if isinstance(value, str):
            # Wildcard chjeck
            if value == 'all':
                self._filter['name'] = 'all'
                return self

            pfilter = []
            try:
                pfilter = le(value)
            except (ValueError, SyntaxError):
                # tc wasn't immediately parseable.
                # This can mean that it was a list/tuple with a string, which gets
                # interpreted as a variable, which causes a malformed string exception.
                # Therefore, need to split and evaluate each part of the list separately
                pass

            if pfilter == []:
                # we get bad names/indexes if we leave parentheses.
                # A dictionary will not add anything to pfilter.
                value = value.strip('([])')
                for item in value.split(','):
                    try:
                        # Transforms string into other python type
                        # Troublesome if someone names a testcase as a valid python type...
                        le_item = le(item)
                        pfilter.append(le_item)
                    except (ValueError, SyntaxError):
                        # It's just a string, but it's also a name(maybe).
                        # Hopefully we're on a filesystem that allows files with identical paths
                        pfilter.append(item)
                if len(pfilter) == 1:
                    # It was a single string.
                    self._filter['name'] = pfilter[0]
                    return self
                elif len(pfilter) == 0:
                    pass

            value = pfilter

        if isinstance(value, int) and (value is not False and value is not True):
            if value < 1:
                raise TypeError("Error, createFilter: non-positive integer " + str(value))
            else:
                self._filter['list'] = [value - 1]
        elif isinstance(value, list) or isinstance(value, tuple):
            if len(value) < 1:
                raise IndexError("Error, createFilter: Index list empty.")
            for i in value:
                if not isinstance(i, int) and not isinstance(i, str):
                    raise TypeError("Error, createFilter: Index list has invalid member: %s" % str(value))
            self._filter['list'] = [x - 1 for x in value if isinstance(x, int)]
            self._filter['list'].extend([x for x in value if isinstance(x, str)])
        elif value is None:
            raise TypeError("tc filter cannot be None")
        else:
            # In case someone calls with NoneType or anything else
            raise TypeError("Unrecognised type for tc filter. tc must be int, str, list or tuple")
        return self

    def status(self, status):
        """
        Add status filter.

        :param status: filter value
        :return: TestcaseFilter
        """
        return self._add_filter_key("status", status)

    def group(self, group):
        """
        Add group filter.

        :param group: Filter value
        :return: TestcaseFilter
        """
        return self._add_filter_key("group", group)

    def testtype(self, testtype):
        """
        Add type filter.

        :param testtype: Filter value
        :return: TestcaseFilter
        """
        return self._add_filter_key("type", testtype)

    def subtype(self, subtype):
        """
        Add subtype filter.

        :param subtype: Filter value
        :return: TestcaseFilter
        """
        return self._add_filter_key("subtype", subtype)

    def component(self, component):
        """
        Add component filter.

        :param component: Filter value
        :return: TestcaseFilter
        """
        return self._add_filter_key("comp", component)

    def feature(self, feature):
        """
        Add feature filter.

        :param feature: Filter value
        :return: TestcaseFilter
        """
        return self._add_filter_key("feature", feature)

    def platform(self, platform):
        """
        Add platform filter.

        :param platform: Filter value
        :return: TestcaseFilter
        """
        return self._add_filter_key("platform", platform)

    def get_filter(self):
        return self._filter

    def match(self, testcase, tc_index):
        """
        Match function. Matches testcase information with this filter.

        :param testcase: TestcaseContainer instance
        :param tc_index: Index of testcase in list
        :return: True if all filter fields were successfully matched to information.
        False otherwise.
        """
        list_ok = False
        testcase = testcase.get_infodict()
        if 'list' in self._filter.keys() and self._filter['list']:
            for index in self._filter['list']:
                if isinstance(index, int):
                    if index < 0:
                        raise TypeError(
                            "Error, filterTestcases: "
                            "index list contained non-positive integer: %s" % self._filter['list'])
                    if index == tc_index:
                        list_ok = True
                        break
                elif isinstance(index, str):
                    if testcase['name'] == index:
                        list_ok = True
                        break
                else:
                    raise TypeError("Error, filterTestcases: "
                                    "index list contained non-integer: '%s'" % self._filter['list'])
            if not list_ok:
                return False
        else:
            list_ok = True

        group_ok = False
        if 'group' in self._filter.keys() and self._filter['group']:
            group = self._filter['group'].split(os.sep)
            group = [x for x in group if len(x)] # Remove empty members
            if len(group) == 1:
                group = self._filter['group'].split(',')
            tcgroup = testcase['group'].split(os.sep)
            for member in group:
                if member not in tcgroup:
                    group_ok = False
                    break
                else:
                    group_ok = True
        else:
            group_ok = True

        platform_ok = True
        if 'platform' in self._filter.keys() and self._filter['platform']:
            platforms = self._filter['platform']
            platforms = platforms if isinstance(platforms, list) else [platforms]
            tcplatforms = testcase['allowed_platforms']
            if tcplatforms:
                for member in platforms:
                    if member not in tcplatforms:
                        platform_ok = False
                    else:
                        platform_ok = True
                        break

        rest_ok = True
        keys = ['status', 'type', 'subtype', 'comp', 'name', 'feature']
        for key in keys:
            if key in self._filter.keys() and self._filter[key]:
                # Possible that string comparison can cause encoding comparison error.
                # In the case where the caseFilter is 'all', the step is skipped
                if key == 'name' and self._filter[key] == 'all':
                    continue
                if isinstance(testcase[key], list):
                    if isinstance(self._filter[key], str) and self._filter[key] not in testcase[key]:
                        rest_ok = False
                        break
                    if isinstance(self._filter[key], list):
                        one_found = False
                        for k in self._filter[key]:
                            if k in testcase[key]:
                                one_found = True
                                break
                        if not one_found:
                            rest_ok = False
                            break
                elif isinstance(testcase[key], str):
                    if isinstance(self._filter[key], str) and testcase[key] != self._filter[key]:
                        rest_ok = False
                        break
                    elif isinstance(self._filter[key], list) and testcase[key] not in self._filter[key]:
                        rest_ok = False
                        break

        return list_ok and group_ok and rest_ok and platform_ok

    def _add_filter_key(self, key, value):
        if not key or not value:
            return self
        if not isinstance(value, str):
            raise TypeError("createFilter: filter argument {} not string.")
        if len(value.split(",")) > 1:
            self._filter[key] = value.split(",")
        else:
            self._filter[key] = value
        return self


"""

Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.


TestSuite module, containing the SuiteException exception and the TestSuite class

TestSuite class is a representation of a runnable suite of test cases.

SuiteException is an Exception that is raised by TestSuite when it needs to exit with a critical
failure.

"""
# pylint: disable=too-many-branches,too-many-arguments

import json
import os
import sys

from icetea_lib.Result import Result
from icetea_lib.ResultList import ResultList
from icetea_lib.TestSuite.TestcaseContainer import TestStatus, DummyContainer
from icetea_lib.TestSuite.TestcaseList import TestcaseList
from icetea_lib.TestSuite.TestcaseFilter import TestcaseFilter


class SuiteException(Exception):
    """
    Raised when something goes wrong with suite creation
    or other operations performed here.
    """
    pass


class TestSuite(object):
    def __init__(self, logger=None, cloud_module=None, args=None):
        self.logger = logger
        if logger is None:
            import logging
            self.logger = logging.getLogger("TestSuite")
            if not self.logger.handlers:
                self.logger.addHandler(logging.StreamHandler())
                self.logger.setLevel(logging.INFO)
        self.args = args
        self.cloud_module = cloud_module
        self._testcases = []
        self._default_configs = {}
        self.status = TestStatus.PENDING
        self._results = ResultList()
        self._create_tc_list()

    def __len__(self):
        return len(self._testcases)

    def get_testcases(self):
        """
        Return internal list of TestcaseContainers
        """
        return self._testcases

    def get_tcnames(self):
        """
        Return list of names of all test cases in this Suite.

        :return: list
        """
        return [tc.get_name() for tc in self._testcases]

    def run(self):
        """
        Test runner
        """
        self.status = TestStatus.RUNNING
        self.logger.info("Starting suite.")
        i = 0
        repeats = int(self.args.repeat) if self.args.repeat and int(self.args.repeat) >= 2 else 1
        repeat = 1
        self.logger.debug("Test suite repeats: %i", repeats)
        while repeat <= repeats:

            self.logger.info("Starting repeat %i of %i", repeat, repeats)
            repeat += 1
            for test in self._testcases:
                self.logger.debug("Starting next test case: %s", test.get_name())
                iterations = self.get_default_configs().get('iteration', 1)
                if iterations == 0:
                    continue
                iteration = 0
                while iteration < iterations:
                    self.logger.info("Iteration %i of %i", iteration + 1, iterations)
                    retries = self.get_default_configs().get("retryCount", 0)
                    self.logger.debug("Test case retries: %i", retries)
                    retryreason = self.get_default_configs().get("retryReason", "inconclusive")
                    iteration += 1
                    if self.args.forceflash_once:
                        self.args.forceflash = i == 0
                        self.logger.debug("Forceflash_once set: Forceflash is %s",
                                          self.args.forceflash)
                        i += 1
                    try:
                        result, retries, repeat, iteration = self._run_testcase(test,
                                                                                retries,
                                                                                repeat,
                                                                                repeats,
                                                                                iteration,
                                                                                iterations,
                                                                                retryreason)
                    except KeyboardInterrupt:
                        self.logger.error("Test run aborted.")
                        self.status = TestStatus.FINISHED
                        return self._results
                    self._upload_results(result)
                if result and result.get_verdict() not in ['pass',
                                                           'skip'] and self.args.stop_on_failure:
                    break
        self.status = TestStatus.FINISHED
        i += 1
        return self._results

    def _run_testcase(self, test, retries, repeat, repeats, iteration, iterations, retryreason):
        """
        Internal runner for handling a single test case run in the suite.
        Repeats and iterations are handled outside this function.

        :param test: TestcaseContainer to be run
        :param retries: Amount of retries desired
        :param repeat: Current repeat index
        :param repeats: Total amount of repeats
        :param iteration: Current iteration index
        :param iterations: Total number of iterations
        :param retryreason: suite related parameter for which test verdicts to retry.
        :return: (Result, retries(int), repeat(int), iteration(int))
        """
        result = None
        while True:
            try:
                self.logger.debug("Starting test case run.")
                result = test.run(forceflash=self.args.forceflash)
                result.retries_left = retries
            except KeyboardInterrupt:
                self.logger.info("User aborted test run")
                iteration = iterations
                repeat = repeats + 1
                raise
            if result is not None:

                # Test had required attributes and ran succesfully or was skipped.
                # Note that a fail *during* a testcase run will still be reported.
                if not isinstance(result, ResultList):
                    result.build_result_metadata(args=self.args)
                self._results.append(result)
                if self.args.stop_on_failure and result.get_verdict() not in ['pass', 'skip']:
                    # Stopping run on failure,
                    self.logger.info("Test case %s failed or was inconclusive, "
                                     "stopping run.\n", test.get_name())
                    repeat = repeats + 1
                    iteration = iterations + 1
                    break
                if result.get_verdict() == 'pass':
                    self.logger.info("Test case %s passed.\n", test.get_name())
                    break
                if result.get_verdict() == 'skip':
                    iteration = iterations
                    result.retries_left = 0
                    self.logger.info("Test case %s skipped.\n", test.get_name())
                    break
                elif retries > 0:
                    if retryreason == "includeFailures" or (retryreason == "inconclusive"
                                                            and result.inconclusive):
                        self.logger.error("Testcase %s failed, %d "
                                          "retries left.\n", test.get_name(), retries)
                        retries -= 1
                        self._upload_results(result)
                        continue
                    else:
                        result.retries_left = 0
                        break
                else:
                    self.logger.error("Test case %s failed, No retries left.\n",
                                      test.get_name())
                    break
        return result, retries, repeat, iteration

    def _upload_results(self, result):
        """
        Upload result to cloud.

        :param result: Result object
        :return: Nothing
        """
        if self.cloud_module:
            self.logger.debug("Uploading results to DB.")
            if isinstance(result, Result):
                self._upload_result(result)
            elif isinstance(result, ResultList):
                for result_item in result:
                    self._upload_result(result_item)

    def _upload_result(self, result_object):
        if not result_object.uploaded:
            response_data = self.cloud_module.send_result(result_object)
            if response_data:
                data = response_data
                self.logger.info("Results sent to the server. ID: %s", data.get('_id'))
                result_object.uploaded = True


    def get_default_configs(self):
        """
        Get suite default configs
        """
        return self._default_configs

    def get_results(self):
        """
        Return results
        """
        return self._results

    def list_testcases(self):
        """
        List all test cases in this Suite in a neat table.

        :return: PrettyTable
        """
        testcases = []
        try:
            if self.args.json:
                self._create_json_objects(testcases)
                if self.args.export:
                    self._create_suite_file(testcases, self.args.export)
                return json.dumps(testcases)
            else:
                self._create_rows_for_table(testcases)
                from prettytable import PrettyTable
                table = PrettyTable(
                    ["Index", "Name", "Status", "Type", "Subtype", "Group", "Component",
                     "Feature", "Allowed platforms"])
                table.align["Index"] = "l"
                for row in testcases:
                    table.add_row(row)
                return table
        except TypeError:
            self.logger.error("Error, print_list_testcases: error during iteration.")
            return

    def _create_suite_file(self, testcases, suite_name):  # pylint: disable=no-self-use
        base_json = dict()
        base_json["default"] = {}
        base_json["testcases"] = []
        for testcase in testcases:
            base_json["testcases"].append({"name": testcase["name"]})
        with open(suite_name, "w") as filehandle:
            filehandle.write(json.dumps(base_json))

    def _create_json_objects(self, testcases):
        for testcase in self._testcases:
            info = testcase.get_instance_config()
            testcases.append(info)

        return testcases

    def _create_rows_for_table(self, rows):
        index = 0
        for testcase in self._testcases:
            info = testcase.get_infodict()
            try:
                index += 1
                grp = info.get('group')
                if grp:
                    group = os.sep.join(info.get('group').split(os.sep)[1:])
                    if not group:
                        group = "no group"
                else:
                    group = "no group"
                rows.append([index, info.get('name'), info.get('status'),
                             info.get('type'), info.get('subtype'),
                             group, info.get('comp'), info.get('feature'),
                             info.get("allowed_platforms")])
            except KeyError:
                self.logger.error("Error, printListTestcases: Testcase list item with "
                                  "index %d missing attributes.", index)

    def update_testcases(self):
        """
        Update test cases of this Suite from cloud.

        :return: Nothing
        """
        if not self.cloud_module:
            self.logger.error("Cloud module has not been initialized! "
                              "Skipping testcase update.")
            return False
        else:
            for testcase in self._testcases:
                try:
                    tc_instance = testcase.get_instance()
                    self.cloud_module.update_testcase(tc_instance.config)
                except Exception as err:  # pylint: disable=broad-except
                    self.logger.error(err)
                    self.logger.debug("Invalid TC: " + testcase.tcname)

    @staticmethod
    def get_suite_files(path):
        """
        Static method for finding all suite files in path.

        :param path: Search path
        :return: List of json files.
        """
        return_list = []
        if not isinstance(path, str):
            return return_list
        if not os.path.exists(path):
            return return_list
        for _, _, files in os.walk(path):
            for fil in sorted(files):
                _, extension = os.path.splitext(fil)
                if extension != '.json':
                    continue
                return_list.append(fil)
        return return_list

    def _create_tc_list(self):
        """
        Parses testcase metadata from suite file or from testcase list in args.
        Sets TestSuite status to 'parsed' to indicate that it has not yet been prepared.

        :raises: SuiteException
        """
        suite = None
        if self.args.suite:
            if os.path.exists(os.path.abspath(self.args.suite)):
                # If suite can be found using just the suite argument, we use that.
                suitedir, filename = os.path.split(os.path.abspath(self.args.suite))
            elif os.path.exists(self.args.suitedir):
                suitedir = self.args.suitedir
                # We presume that this is just the filename, or a path relative to the suitedir.
                filename = self.args.suite
            else:
                raise SuiteException("Suite creation from file failed. Unable to determine suite "
                                     "directory. Check --suitedir and --suite.")
            suite = self._load_suite_file(filename, suitedir)
            if not suite:
                raise SuiteException("Suite creation from file failed. "
                                     "Check your suite file format, path and access rights.")
            self._default_configs = suite.get("default", {})
            tcnames = []
            for i, testcase in enumerate(suite.get("testcases")):
                tcnames.append(str(testcase.get("name")))
            testcases = self._get_suite_tcs(self.args.tcdir, tcnames)
            if not testcases:
                raise SuiteException("Suite creation failed: Unable to find or filter testcases.")
            self._testcases = testcases
            self._print_search_errors()
            if len(testcases) != len(suite.get("testcases")):
                raise SuiteException("Suite creation from file failed: "
                                     "Number of requested testcases does not match "
                                     "amount of found testcases.")

            for i, testcase in enumerate(suite.get("testcases")):
                suiteconfig = testcase.get("config")
                self._testcases.get(i).set_suiteconfig(suiteconfig)
        else:
            tclist = self._load_suite_list()
            if tclist is False:
                raise SuiteException("Suite creation failed.")
            self._testcases = tclist
            if self.args.tc and self.args.tc != "all":
                self._print_search_errors()
            elif self._testcases.search_errors:
                self.logger.error("Failed import the following modules during test case search:")
                for item in self._testcases.search_errors:
                    self.logger.error("%s: %s", item["module"], item["error"])
        self.logger.info("Suite creation complete.")
        self._prepare_suite()

    def _print_search_errors(self):
        for testcase in self._testcases:
            if isinstance(testcase, DummyContainer):
                self.logger.error("Some test cases were not found.")
                for item in self._testcases.search_errors:
                    self.logger.error("%s: %s", item["module"], item["error"])

    def _prepare_suite(self):
        """
        Prepares parsed testcases for running.

        :raises: SyntaxError, SuiteException
        """
        for i, testcase in enumerate(self._testcases):
            try:
                self._prepare_testcase(testcase)
            except (TypeError, ImportError, ValueError) as err:
                raise SuiteException("Test case preparation failed for "
                                     "test case {}: {}".format(i, err))
            except SyntaxError:
                pass
                if self.args.list:
                    pass
                else:
                    raise
            testcase.status = TestStatus.READY
        self.logger.info("Test cases prepared.")
        self.status = TestStatus.READY

    def _get_suite_tcs(self, tcdir, testcases):
        """
        Generate a TestcaseList from a Suite.

        :param tcdir: Test case directory
        :param testcases: Names of testcases.
        :return: TestcaseList or None
        """
        if not os.path.isdir(tcdir):
            self.logger.error("Test case directory does not exist!")
            return None
        self.logger.info("Importing testcases for filtering")
        abs_tcpath = os.path.abspath(tcdir)
        sys.path.append(abs_tcpath)
        tclist = TestcaseList(logger=self.logger)
        tclist.import_from_path(abs_tcpath)
        if not tclist:
            self.logger.error("Error, runSuite: "
                              "Could not find any python files in given testcase dirpath")
            return None
        try:
            filt = TestcaseFilter().tc(testcases)
        except (TypeError, IndexError):
            self.logger.error("Error: Failed to create testcase filter for suite.")
            return None
        self.logger.info("Filtering testcases")
        if testcases == "all":
            testcases = None
        final_tclist = tclist.filter(filt, testcases)
        if not final_tclist:
            self.logger.error("Error, create_suite: "
                              "Specified testcases not found in %s.", abs_tcpath)
            return None
        return final_tclist

    def _prepare_testcase(self, testcase):
        """
        Run some preparatory commands on a test case to prep it for running.

        :param testcase: TestcaseContainer
        :return: Nothing
        """
        testcase.validate_tc_instance()
        testcase.merge_tc_config(self._default_configs)
        if testcase.get_suiteconfig():
            testcase.merge_tc_config(testcase.get_suiteconfig())
        testcase.set_final_config()
        testcase.validate_tc_instance()

    def _load_suite_file(self, name, suitedir):
        """
        Load a suite file from json to dict.

        :param name: Name of suite
        :param suitedir: Path to suite
        :return: Dictionary or None
        """
        self.logger.info("Loading suite from file")
        if not isinstance(name, str):
            self.logger.error("Error, load_suite: Suite name not a string")
            return None
        filename = name if name.split('.')[-1] == 'json' else name + '.json'
        filepath = os.path.join(suitedir, filename)

        suite = None
        if not os.path.exists(filepath):
            if self.cloud_module:
                suite = self.cloud_module.get_suite(name)
            else:
                self.logger.error("Error, load_suite_file: "
                                  "Suite file not found and cloud module not defined.")
                return None
            if not suite:
                self.logger.error("Error, load_suite_file: "
                                  "Suite file not found locally or in cloud.")
            return suite
        try:
            with open(filepath) as fil:
                suite = json.load(fil)
                return suite
        except IOError:
            self.logger.error("Error, load_suite_file: "
                              "Test suite %s cannot be read.", name)
        except ValueError:
            self.logger.error("Error, load_suite_file: "
                              "Could not load test suite. No JSON object could be decoded.")
        return None

    def _load_suite_list(self):
        """
        Generate a TestcaseList from command line filters.

        :return: TestcaseList or False
        """
        self.logger.info("Generating suite from command line.")
        args = self.args
        filt = TestcaseFilter()
        testcase = args.tc if args.tc else "all"
        try:
            filt = TestcaseFilter().tc(testcase)
            filt.status(args.status).group(args.group).testtype(args.testtype)
            filt.subtype(args.subtype).component(args.component).feature(args.feature)
            filt.platform(args.platform_filter)
        except (TypeError, IndexError):
            self.logger.exception("Filter creation failed.")
            return False

        self.logger.info("Importing testcases for filtering")
        if not os.path.isdir(args.tcdir):
            self.logger.error("Test case directory does not exist!")
            return False
        abs_tcpath = os.path.abspath(args.tcdir)
        sys.path.append(abs_tcpath)
        tclist = TestcaseList(self.logger)
        tclist.import_from_path(abs_tcpath)
        if not tclist:
            self.logger.error("Could not find any python files in given path")
            return False
        self.logger.info("Filtering testcases")
        if filt.get_filter()["list"] is not False:
            if isinstance(filt.get_filter()["list"], list):
                testcases = filt.get_filter()["list"]
            else:
                testcases = None
        else:
            testcases = None
        final_tclist = tclist.filter(filt, testcases)
        if not final_tclist:
            self.logger.error("Error, create_suite: "
                              "Specified testcases not found in %s.", abs_tcpath)
        return final_tclist



# pylint: disable=missing-docstring,redundant-unittest-assert

"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest
import mock

import icetea_lib.tools.asserts as asserts
from icetea_lib.TestStepError import TestStepFail


class MockBench(object):
    def __init__(self):
        pass

    def command(self):
        pass

    def logger(self):
        pass


class AssertTestcase(unittest.TestCase):

    def setUp(self):
        self.asserts = asserts

    def test_assert_booleans(self):
        with self.assertRaises(TestStepFail):
            self.asserts.assertTrue(False, "False was not True!")
        with self.assertRaises(TestStepFail):
            self.asserts.assertFalse(True)

        try:
            self.asserts.assertTrue(True, "True was somehow False?!")
            self.asserts.assertTrue([1, 2])
            self.assertTrue(True, "No fail was raised.")
        except TestStepFail:
            self.assertTrue(False, "TestStepFail was raised! ")
        try:
            self.asserts.assertFalse(False)
            self.asserts.assertFalse([])
            self.asserts.assertFalse({})
            self.assertTrue(True, "No fail was raised.")
        except TestStepFail:
            self.assertTrue(False, "TestStepFail was raised! ")

    def test_assert_nones(self):
        with self.assertRaises(TestStepFail):
            self.asserts.assertNone(1)
        with self.assertRaises(TestStepFail):
            self.asserts.assertNotNone(None)

        try:
            self.asserts.assertNone(None)
            self.asserts.assertNotNone(1)
        except TestStepFail:
            self.assertTrue(False, "TestStepFail was raised!")

    def test_assert_equals(self):
        with self.assertRaises(TestStepFail):
            self.asserts.assertEqual(1, 2)
        with self.assertRaises(TestStepFail):
            self.asserts.assertNotEqual(1, 1)

        try:
            self.asserts.assertEqual(1, 1)
            self.asserts.assertNotEqual(1, 2)
        except TestStepFail:
            self.assertTrue(False, "TestStepFail was raised!")

    def test_assert_json_contains(self):
        with self.assertRaises(TestStepFail):
            self.asserts.assertJsonContains('{"test": "key"}', "test2")

        with self.assertRaises(TestStepFail):
            self.asserts.assertJsonContains("{'test': 'key'}", "test")

        with self.assertRaises(TestStepFail):
            self.asserts.assertJsonContains(None, "test")

        try:
            self.asserts.assertJsonContains('{"test": "key"}', 'test')
        except TestStepFail:
            self.assertTrue(False, 'Key test was not contained in {"test": "key"}?')

    def test_assert_dut_trace_contains(self):
        mock_bench = mock.MagicMock()
        mock_bench.verify_trace = mock.MagicMock(side_effect=[True, False])
        self.asserts.assertDutTraceContains(1, "message_found", mock_bench)
        with self.assertRaises(TestStepFail):
            self.asserts.assertDutTraceContains(1, "message_not_found", mock_bench)

    def test_assert_dut_trace_does_not_contain(self):  # pylint: disable=invalid-name
        mock_bench = mock.MagicMock()
        mock_bench.verify_trace = mock.MagicMock(side_effect=[True, False])
        with self.assertRaises(TestStepFail):
            self.asserts.assertDutTraceDoesNotContain(1, "message_found", mock_bench)
        self.asserts.assertDutTraceDoesNotContain(1, "message_not_found", mock_bench)


if __name__ == '__main__':
    unittest.main()



"""
Copyright 2017-2018 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import os
import sys
from distutils.core import setup
from setuptools import find_packages
from setuptools.command.install import install


DESCRIPTION = "Icetea - test framework"
OWNER_NAMES = "Jussi Vatjus-Anttila"
OWNER_EMAILS = "jussi.vatjus-anttila@arm.com"
VERSION = "1.2.1"


def read(fname):
    """
    Utility function to cat in a file
    :param fname: filename
    :return: file content as a String
    """
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


INSTALL_REQUIRES = [
    "prettytable==0.*",
    "requests",
    "yattag==1.*",
    "pyserial>2.5",
    "jsonmerge==1.*",
    "jsonschema<3.0.0",
    "mbed-ls>=1.5.1,==1.*",
    "semver==2.*",
    "mbed-flasher>=0.10.1,==0.10.*",
    "six==1.*"
]
TEST_REQUIRES = [
    "coverage==4.*",
    "mock==2.*",
    "sphinx==1.*",
    "lxml",
    "pylint==1.*",
    "astroid==1.*"
]
# todo move pyshark as optional dep:
# pyshark-legacy; python_version < "3.0"
# pyshark; python_version >= "3.5"
if sys.version_info[0] < 3:
    INSTALL_REQUIRES.append("pyshark-legacy")
else:
    INSTALL_REQUIRES.append("pyshark")


class VerifyVersionCommand(install):
    """
    Custom command to verify that the git tag matches our version
    """
    description = "verify that the git tag matches our version"

    def run(self):
        is_ci = os.getenv("CIRCLECI")
        if is_ci:
            tag = os.getenv("CIRCLE_TAG")
            version = "v" + VERSION
            if tag != version:
                info = "Git tag: {0} does not match the"\
                       "version of this app: {1}".format(tag, version)
                sys.exit(info)
        # else: you are your own - please do not publish any releases without tag!


setup(name="icetea",
      version=VERSION,
      description=DESCRIPTION,
      long_description=read("README.md"),
      long_description_content_type='text/markdown',
      author=OWNER_NAMES,
      author_email=OWNER_EMAILS,
      maintainer=OWNER_NAMES,
      maintainer_email=OWNER_EMAILS,
      url="https://github.com/ARMmbed/icetea.git",
      packages=find_packages(include=["icetea_lib.*", "icetea_lib"]),
      data_files=[("icetea_lib", ["icetea_lib/tc_schema.json"])],
      include_package_data=True,
      keywords="armbed mbed-os mbed-cli ci framework testing automation",
      license="(R) ARM",
      tests_require=TEST_REQUIRES,
      test_suite="test",
      entry_points={
          "console_scripts": [
              "icetea=icetea_lib:icetea_main"
          ]
      },
      install_requires=INSTALL_REQUIRES,
      classifiers=[
          "Programming Language :: Python :: 2.7",
          "Programming Language :: Python :: 3.5",
          "Programming Language :: Python :: 3.6",
          "Programming Language :: Python :: 3.7",
          "License :: OSI Approved :: Apache Software License",
          "Operating System :: OS Independent",
      ],
      cmdclass={
          "verify": VerifyVersionCommand,
      }
     )



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import logging


class ParserManager(object):
    """
    ParserManager class. This is a manager for all parsers and this is the entry point for test
    cases to handle response parsing. PluginManager appends all parsers to this class and tests
    call the parse-method to parse received responses.
    """

    def __init__(self, logger=None):
        self.parsers = {}
        self.logger = logger
        if self.logger is None:
            self.logger = logging.getLogger("ParserManager")
            self.logger.addHandler(logging.StreamHandler().setLevel(logging.WARNING))
            self.logger.setLevel(logging.WARNING)

    def has_parser(self, parser):
        """
        Returns True if given parser found in managed parsers.

        :param parser: Name of parser to look for
        :return: Boolean
        """
        return True if parser in self.parsers else False

    def add_parser(self, parser_name, parser_func):
        """
        Add new parser function for specific name.

        :param parser_name: Name of parser to add
        :param parser_func: Callable function to call when parsing responses
        :return: Nothing
        """
        self.parsers[parser_name] = parser_func

    # parse response
    def parse(self, *args, **kwargs):  # pylint: disable=unused-argument
        """
        Parse response.

        :param args: List. 2 first items used as parser name and response to parse
        :param kwargs: dict, not used
        :return: dictionary or return value of called callable from parser.
        """
        #pylint: disable=W0703
        cmd = args[0]
        resp = args[1]
        if cmd in self.parsers:
            try:
                return self.parsers[cmd](resp)
            except Exception as err:
                print(err)
        return {}



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

__author__ = 'jaakuk03'
import unittest
import mock
import sys
import os
import re
import icetea_lib.tools.tools as tools
from pkg_resources import DistributionNotFound

class TestClass():
    def __init__(self):
        self.test = True


def testingfunction(arg1, arg2):  # pylint: disable=unused-argument
    pass


class TestTools(unittest.TestCase):
    def test_loadClass_Success(self):
        sys.path.append(os.path.dirname(__file__))
        # Test that loadClass can import a class that is initializable
        module = tools.load_class("test_tools.TestClass", verbose=False, silent=True)
        self.assertIsNotNone(module)
        moduleInstance = module()
        self.assertTrue(moduleInstance.test)
        del moduleInstance

    def test_loadClass_Fail(self):
        with self.assertRaises(Exception):
            tools.load_class('testbase.level1.Testcase', verbose=False, silent=True)

        self.assertIsNone(tools.load_class('', verbose=False, silent=True))
        self.assertIsNone(tools.load_class(5, verbose=False, silent=True))
        self.assertIsNone(tools.load_class([], verbose=False, silent=True))
        self.assertIsNone(tools.load_class({}, verbose=False, silent=True))

    def test_combine_urls(self):
        self.assertEquals(tools.combine_urls("/path/one/", "path2"), "/path/one/path2")
        self.assertEquals(tools.combine_urls("/path/one", "path2"), "/path/one/path2")
        self.assertEquals(tools.combine_urls("/path/one/", "/path2"), "/path/one/path2")
        self.assertEquals(tools.combine_urls("/path/one", "/path2"), "/path/one/path2")

    def test_hex_escape_tester(self):
        failing_message = "\x00\x00\x00\x00\x00\x00\x01\xc8"
        success_message = "\\x00\\x00\\x00\\x00\\x00\\x00\\x01\\xc8"

        converted = tools.hex_escape_str(failing_message)
        self.assertEqual(converted, success_message)

    def test_get_fw_version(self):
        version = None
        try:
            setup_path = os.path.abspath(os.path.dirname(__file__)+'/..')
            with open(os.path.join(setup_path, 'setup.py')) as setup_file:
                lines = setup_file.readlines()
                for line in lines:
                    m = re.search(r"VERSION = \"([\S]{5,})\"", line)
                    if m:
                        version = m.group(1)
                        break
        except Exception as e:
            pass

        v = tools.get_fw_version()
        self.assertEqual(v, version)

        with mock.patch("icetea_lib.tools.tools.require") as mocked_require:
            mocked_require.side_effect = [DistributionNotFound]
            v = tools.get_fw_version()
            self.assertEqual(v, version)

    def test_check_int(self):
        integer = "10"
        prefixedinteger = "-10"
        decimal = "10.1"
        self.assertTrue(tools.check_int(integer))
        self.assertTrue(tools.check_int(prefixedinteger))
        self.assertFalse(tools.check_int(decimal))
        self.assertFalse(tools.check_int(1))

    def test_remove_empty_from_dict(self):
        d = {"test": "val", "test2": None}
        d = tools.remove_empty_from_dict(d)
        self.assertDictEqual({"test": "val"}, d)

    def test_setordelete(self):
        d = {"test": "val",
             "test2": "val2"}
        tools.set_or_delete(d, "test", "val3")
        self.assertEqual(d.get("test"), "val3")
        tools.set_or_delete(d, "test2", None)
        self.assertTrue("test2" not in d)

    def test_getargspec(self):
        expected_args = ["arg1", "arg2"]
        spec = tools.getargspec(testingfunction)
        for item in expected_args:
            self.assertTrue(item in spec.args)

    def test_strip_escape(self):
        test_data_no_escapes = "aaaa"
        self.assertEqual(tools.strip_escape(test_data_no_escapes), "aaaa")
        test_data_encoded = "aaaa".encode("utf-8")
        self.assertEqual(tools.strip_escape(test_data_encoded), "aaaa")
        mock_str = mock.MagicMock()
        mock_str.decode = mock.MagicMock(side_effect=[UnicodeDecodeError])
        with self.assertRaises(TypeError):
            tools.strip_escape(mock_str)



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

AllocationContext module, cotnains two classes that represent
allocation information of resources allocated for a test case.

These classes implement storing of Allocated resources as well as conversion of allocated
resources into Dut objects usable by test cases.
"""

import os

try:
    # Python 2.7 compatible import.
    from Queue import Queue
except ImportError:
    # Python 3 has renamed Queue to queue.
    from queue import Queue
from multiprocessing.pool import ThreadPool

from icetea_lib.DeviceConnectors.Dut import DutConnectionError
from icetea_lib.ResourceProvider.exceptions import ResourceInitError
from icetea_lib.ResourceProvider.Allocators.exceptions import AllocationError
from icetea_lib.build.build import Build


class AllocationContext(object):
    """
    Class AllocationContext

    Contains allocation and resource_id:s used to define an allocated resource.
    Also contains supplementary data in _alloc_data dictionary.
    """

    def __init__(self, resource_id=None, alloc_id=None, alloc_data=None):
        self._resource_id = resource_id
        self._alloc_id = alloc_id
        self._alloc_data = alloc_data if alloc_data else {}

    def get(self, key):
        """
        Getter for configuration dictionary items

        :param key: Key to look for in configuration.
        :param default: Default value to be returned if key is not found in configuration.
                Default is None
        :return: value of configuration[key] if key exists in configuration, else value of default.
        """
        return self._alloc_data.get(key)

    def set(self, key, value):
        """
        Setter for configuration values

        :param key:
        :param value:
        :return: Nothing
        """
        self._alloc_data[key] = value

    def get_alloc_data(self):
        """
        Get allocation data dictionary

        :return: Allocation data (dictionary)
        """
        return self._alloc_data

    @property
    def alloc_id(self):
        """
        Getter for alloc_id

        :return: alloc_id
        """
        return self._alloc_id

    @property
    def resource_id(self):
        """
        Getter for resource_id

        :return: resource_id
        """
        return self._resource_id

    def __getitem__(self, item):
        """
        __getitem__ implementation for AllocationContext
        enables context[item]

        :param item: item to return
        :return: self._alloc_data[item] or None if item does not exist in self._alloc_data
        """
        return self._alloc_data.get(item, None)

    def __setitem__(self, key, value):
        """
        __setitem__ implementation for AllocationContext.
        Replaces item in list space key with value.

        :param key: Name of context data item to replace/add
        :param value: Value to replace context data item with
        :return: Nothing
        """
        self._alloc_data[key] = value


class AllocationContextList(object):
    """
    Class AllocationContextList

    Is used for storing and handling Duts after they have been allocated.
    Contains methods to iterate over AllocationContext objects and initialize duts from
    AllocationContext objects.
    """

    def __init__(self, logger=None):
        self._allocation_contexts = []
        self.logger = logger
        self.duts = []
        self.dutinformations = []
        self._resource_configuration = None
        self._dut_initialization_functions = {}

    def __len__(self):
        """
        len implementation for AllocationContextList

        :return: len(self._allocation_contexts)
        """
        return len(self._allocation_contexts)

    def __iter__(self):
        """
        Implementation of __iter__ to allow for item in list loops

        :return: iterator to self._allocation_contexts
        """
        return iter(self._allocation_contexts)

    def __getitem__(self, item):
        """
        __getitem__ implementation for AllocationContextList
        enables list[index]

        :param item: index to return
        :return: self._allocation_contexts[item]
        :raises: TypeError if item is not integer.
        :raises: IndexError if item < 0 or item > len(self._allocation_contexts)
        """
        if not isinstance(item, int):
            raise TypeError("AllocationContextList get expects an integer index")
        if len(self._allocation_contexts) <= item:
            raise IndexError("list getitem out of bounds")
        elif item < 0:
            raise IndexError("AllocationContextList get not implemented for negative values.")
        return self._allocation_contexts[item]

    def __setitem__(self, key, value):
        """
        __setitem__ implementation for AllocationContextList.
        Replaces item in list space key with value.

        :param key: Index of list item to replace
        :param value: Value to replace list item with
        :return: Nothing
        :raises: TypeError if key is not an integer.
        :raises: IndexError if key < 0 or key > len(self._allocation_contexts)
        """
        if not isinstance(key, int):
            raise TypeError("AllocationContextList set expects an integer index")
        if len(self._allocation_contexts) <= key:
            raise IndexError("list setitem out of bounds")
        elif key < 0:
            raise IndexError("AllocationContextList set not implemented for negative indexes.")
        self._allocation_contexts[key] = value

    def set_dut_init_function(self, dut_type, fnctn):
        """
        Setter for dut initialization function

        :param dut_type: Dut type
        :param fnctn: callable
        :return: Nothing
        """
        self._dut_initialization_functions[dut_type] = fnctn

    def get_dut_init_function(self, dut_type):
        """
        Get dut initialization function

        :param dut_type: Dut type.
        :return: callable
        """
        return self._dut_initialization_functions.get(dut_type)

    def set_resconf(self, resconf):
        """
        Set resource configuration.

        :param resconf: ResourceConfig object
        :return: Nothing
        """
        self._resource_configuration = resconf

    def get_resconf(self):
        """
        Get resource configuration for this AllocationContextList.

        :return: ResourceConfig
        """
        return self._resource_configuration

    def set_logger(self, logger):
        """
        Set logger.

        :param logger: logging.logger
        :return: Nothing
        """
        self.logger = logger

    def get_duts(self):
        """
        Get list of duts.

        :return: list of Dut objects
        """
        return self.duts

    def get_dutinformations(self):
        """
        Get DutInformation objects of current duts.

        :return: list of DutInformation objects
        """
        return self.dutinformations

    def append(self, alloc_context):
        """
        Appends alloc_context to self._allocation_contexts. No type-checking done here.

        :param alloc_context: AllocationContext object.
        :return: Nothing
        """
        self._allocation_contexts.append(alloc_context)

    # pylint: disable=too-many-statements
    def init_duts(self, args):  # pylint: disable=too-many-locals,too-many-branches
        """
        Initializes duts of different types based on configuration provided by AllocationContext.
        Able to do the initialization of duts in parallel, if --parallel_flash was provided.

        :param args: Argument Namespace object
        :return: list of initialized duts.
        """
        # TODO: Split into smaller chunks to reduce complexity.
        threads = []
        abort_queue = Queue()

        def thread_wrapper(*thread_args, **thread_kwargs):
            """
            Run initialization function for dut

            :param thread_args: arguments to pass to the function
            :param thread_kwargs: keyword arguments, (func: callable, abort_queue: Queue)
            :return: Result of func(*thread_args)
            """
            # pylint: disable=broad-except
            try:
                return thread_kwargs["func"](*thread_args)
            except Exception as error:
                thread_kwargs["abort_queue"].put((thread_args[2], error))

        for index, dut_conf in enumerate(self._allocation_contexts):
            dut_type = dut_conf.get("type")
            func = self.get_dut_init_function(dut_type)
            if func is None:
                continue

            threads.append(((self, dut_conf.get_alloc_data().get_requirements(), index + 1, args),
                            {"func": func, "abort_queue": abort_queue}))

        try:
            thread_limit = len(threads) if args.parallel_flash else 1
            pool = ThreadPool(thread_limit)
            async_results = [pool.apply_async(func=thread_wrapper,
                                              args=t[0], kwds=t[1])
                             for t in threads]
            # Wait for resources to be ready.
            [res.get() for res in async_results]  # pylint: disable=expression-not-assigned

            pool.close()
            pool.join()

            if not abort_queue.empty():
                msg = "Dut Initialization failed, reason(s):"
                while not abort_queue.empty():
                    dut_index, error = abort_queue.get()
                    msg = "{}\nDUT index {} - {}".format(msg, dut_index, error)

                raise AllocationError(msg)

            # Sort duts to same order as in dut_conf_list
            self.duts.sort(key=lambda d: d.index)
            self.dutinformations.sort(key=lambda d: d.index)

        except KeyboardInterrupt:
            msg = "Received keyboard interrupt, waiting for flashing to finish"
            self.logger.info(msg)
            for dut in self.duts:
                dut.close_dut(False)
                dut.close_connection()
                if hasattr(dut, "release"):
                    dut.release()
                dut = None
            raise
        except RuntimeError:
            self.logger.exception("RuntimeError during flashing")
        # ValueError is raised if ThreadPool is tried to initiate with
        # zero threads.
        except ValueError:
            self.logger.exception("No devices allocated")
            raise AllocationError("Dut Initialization failed!")
        except (DutConnectionError, TypeError):
            for dut in self.duts:
                if hasattr(dut, "release"):
                    dut.release()
            raise AllocationError("Dut Initialization failed!")
        finally:
            if pool:
                pool.close()
                pool.join()
        self.logger.debug("Allocated following duts:")
        for dut in self.duts:
            dut.print_info()

        return self.duts

    def open_dut_connections(self):
        """
        Opens connections to Duts. Starts Dut read threads.

        :return: Nothing
        :raises DutConnectionError: if problems were encountered while opening dut connection.
        """
        for dut in self.duts:
            try:
                dut.start_dut_thread()
                if hasattr(dut, "command"):
                    dut.open_dut(dut.command)
                else:
                    dut.open_dut()
            except DutConnectionError:
                self.logger.exception("Failed when opening dut connection")
                dut.close_dut(False)
                dut.close_connection()
                dut = None
                raise

    def check_flashing_need(self, execution_type, build_id, force):
        """
        Check if flashing of local device is required.

        :param execution_type: Should be 'hardware'
        :param build_id: Build id, usually file name
        :param force: Forceflash flag
        :return: Boolean
        """
        binary_file_name = AllocationContextList.get_build(build_id)
        if binary_file_name:
            if execution_type == 'hardware' and os.path.isfile(binary_file_name):
                if not force:
                    #@todo: Make a better check for binary compatibility
                    extension_split = os.path.splitext(binary_file_name)
                    extension = extension_split[-1].lower()
                    if extension != '.bin' and extension != '.hex':
                        self.logger.debug("File ('%s') is not supported to flash, skip it" %(
                            build_id))
                        return False
                    return True
                return True
            else:
                raise ResourceInitError("Given binary %s does not exist" % build_id)
        else:
            raise ResourceInitError("Given binary %s does not exist" % build_id)

    @staticmethod
    def get_build(build_id):
        """
        Gets a file related to build_id.
        """
        try:
            build = Build.init(build_id)
        except NotImplementedError:
            return None
        return build.get_file()



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import os
import unittest

from icetea_lib.TestSuite.TestcaseFilter import TestcaseFilter
from icetea_lib.TestSuite.TestcaseContainer import TestcaseContainer
from icetea_lib.IceteaManager import TCMetaSchema


class TCFilterTestcase(unittest.TestCase):

    def test_create_filter_simple(self):
        filt = TestcaseFilter().tc("test_cmdline")
        self.assertDictEqual(filt._filter, {'status': False, 'group': False,
                                            'name': 'test_cmdline', 'comp': False,
                                            'platform': False, 'list': False, 'subtype': False,
                                            'type': False, 'feature': False})

        with self.assertRaises(TypeError):
            TestcaseFilter().tc(0)
        with self.assertRaises(IndexError):
            TestcaseFilter().tc([])
        with self.assertRaises(TypeError):
            TestcaseFilter().tc(None)
        with self.assertRaises(TypeError):
            TestcaseFilter().tc(True)

        self.assertDictEqual(TestcaseFilter().tc(1)._filter, {'status': False,
                                                              'group': False,
                                                              'name': False,
                                                              'comp': False,
                                                              'platform': False,
                                                              'list': [0],
                                                              'subtype': False,
                                                              'type': False,
                                                              'feature': False})

        self.assertDictEqual(TestcaseFilter().tc([1, 4])._filter, {'status': False,
                                                                   'group': False,
                                                                   'name': False,
                                                                   'comp': False,
                                                                   'platform': False,
                                                                   'list': [0, 3],
                                                                   'subtype': False,
                                                                   'type': False,
                                                                   'feature': False})

    def test_create_filter_complex(self):
        filt = TestcaseFilter()

        filt.tc("test_test")
        filt.component("test_comp")
        filt.group("test_group")
        filt.status("test_status")
        filt.testtype("test_type")
        filt.subtype("test_subtype")
        filt.feature("test_feature")
        filt.platform("test_platform")

        self.assertDictEqual(filt._filter, {"status": "test_status", "group": "test_group", "name": "test_test",
                                            "type": "test_type", "subtype": "test_subtype",
                                            "comp": "test_comp", 'list': False, 'feature': "test_feature",
                                            "platform": "test_platform"})

        with self.assertRaises(TypeError):
            filt.component(2)

    def test_match(self):
        tc = TestcaseContainer.find_testcases("examples.test_cmdline", "." + os.path.sep + "examples", TCMetaSchema().get_meta_schema())
        filt = TestcaseFilter().tc("test_cmdline")
        self.assertTrue(filt.match(tc[0], 0))
        filt.component("cmdline,testcomponent")
        filt.group("examples")
        self.assertTrue(filt.match(tc[0], 0))
        filt.tc("test_something_else")
        self.assertFalse(filt.match(tc[0], 0))
        filt = TestcaseFilter().tc([1])
        self.assertTrue(filt.match(tc[0], 0))
        self.assertFalse(filt.match(tc[0], 1))



if __name__ == '__main__':
    unittest.main()



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

ReportConsole module, contains the ReportConsole class which implements reporting to console.
"""

from prettytable import PrettyTable
from icetea_lib.Reports.ReportBase import ReportBase
from icetea_lib.tools.tools import hex_escape_str


class ReportConsole(ReportBase):
    """
    ReportConsole class, implements generating and printing reports to the console using print
    method and PrettyTable module.
    """
    def __init__(self, results):
        ReportBase.__init__(self, results)

    def generate(self, *args, **kwargs):
        """
        Generates and prints the console report, which consists of a table of test cases ran as
        well as a summary table with passrate, number of test cases and statistics on
        passed/failed/inconclusive/skipped cases.

        :param args: arguments, not used
        :param kwargs: keyword arguments, not used
        :return: Nothing
        """
        # Generate TC result table
        table = PrettyTable(
            ["Testcase", "Verdict", "Fail Reason", "Skip Reason", "platforms", "duration"])
        table.align["Testcase"] = "l"  # Left align
        for result in self.results:
            table.add_row([
                result.get_tc_name(),
                result.get_verdict(),
                hex_escape_str(result.fail_reason)[:60],
                str(result.skip_reason) if result.skipped() else "",
                result.get_dut_models(),
                str(result.duration)])
            # Print to console
        print(table)  # pylint: disable=superfluous-parens

        # Generate Summary table
        table = PrettyTable(['Summary', ''])
        final_verdict = "FAIL"
        if self.summary["fail"] == 0 and self.summary["inconclusive"] == 0:
            final_verdict = "PASS"
        elif self.summary["fail"] == 0 and self.summary["inconclusive"] > 0:
            final_verdict = "INCONCLUSIVE"
        table.add_row(["Final Verdict", final_verdict])
        table.add_row(["count", str(self.summary["count"])])

        table.add_row(["passrate", self.results.pass_rate()])
        if self.summary["pass"] > 0:
            table.add_row(["pass", str(self.summary["pass"])])
        if self.summary["fail"] > 0:
            table.add_row(["fail", str(self.summary["fail"])])
        if self.summary["skip"] > 0:
            table.add_row(["skip", str(self.summary["skip"])])
        if self.summary["inconclusive"] > 0:
            table.add_row(["inconclusive", str(self.summary["inconclusive"])])
        table.add_row(["Duration", self.duration_to_string(self.summary["duration"])])
        # Print to console
        print(table)  # pylint: disable=superfluous-parens



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import uuid
from datetime import datetime
import json


class Seed(object):
    def __init__(self, value, seed_id=None, date=None):
        self.__seed_value = value
        self.__seed_id = seed_id if seed_id else str(uuid.uuid4())
        self.__date = date if date else datetime.utcnow().isoformat()

    def __repr__(self):
        return str(self.__seed_value)

    def __add__(self, other):
        return self.value + other

    def __radd__(self, other):
        return other + self.value

    def __cmp__(self, other):
        return cmp(self.value, other)

    @property
    def value(self):
        return self.__seed_value

    @property
    def seed_id(self):
        return self.__seed_id

    @property
    def date(self):
        return self.__date

    def store(self, filename):
        with open(filename, 'w') as f:
            seed_dict = {"seed_id": self.seed_id, "seed_value": self.value, "date": self.date}
            json.dump(seed_dict, f)

    @staticmethod
    def load(filename):
        with open(filename, 'r') as f:
            return json.load(f)


class SeedInteger(Seed):
    def __iadd__(self, other):
        return SeedInteger(self.value + other)

    @staticmethod
    def load(filename):
        json_obj = Seed.load(filename)
        return SeedInteger(json_obj["seed_value"], json_obj["seed_id"], json_obj["date"])


class SeedString(Seed):
    def __getitem__(self, index):
        return self.value[index]

    def __len__(self):
        return len(self.value)

    def __str__(self):
        return str(self.value)

    def __iter__(self):
        for elem in self.value:
            yield elem

    @staticmethod
    def load(filename):
        json_obj = Seed.load(filename)
        return SeedString(json_obj["seed_value"], json_obj["seed_id"], json_obj["date"])


class SeedStringArray(Seed):
    def __getitem__(self, index):
        return self.value[index]

    def __len__(self):
        return len(self.value)

    def __iter__(self):
        for elem in self.value:
            yield elem

    def store(self, filename):
        with open(filename, 'w') as f:
            seed_dict = {"seed_id": self.seed_id, "seed_value": self.value, "date": self.date}
            json.dump(seed_dict, f, default=lambda array_elem: array_elem.value)

    @staticmethod
    def load(filename):
        json_obj = Seed.load(filename)
        return SeedStringArray(json_obj["seed_value"], json_obj["seed_id"], json_obj["date"])



# pylint: disable=missing-docstring,protected-access,unused-argument

"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


import argparse
import json
import os
import unittest
from jsonmerge import merge

import mock

from icetea_lib.TestSuite.TestSuite import TestSuite, SuiteException, TestStatus
from icetea_lib.TestSuite.TestcaseContainer import DummyContainer
from icetea_lib.Result import Result
from icetea_lib.ResultList import ResultList
from icetea_lib.DeviceConnectors.DutInformation import DutInformation
from icetea_lib.build.build import Build


def mock_get_suite_tcs(arg1, arg2):
    return None

def mock_get_local_tcs(tcpath):
    return []

def mock_create_filter(tc):
    return -1

def mock_parse_local_tcs(arg1, arg2):
    return []


class TestSuiteTestcase(unittest.TestCase):

    def setUp(self):
        with open(os.path.join("./icetea_lib", 'tc_schema.json')) as data_file:
            self.tc_meta_schema = json.load(data_file)
        testpath = os.path.dirname(os.path.abspath(__file__))
        self.testdir = os.path.join(testpath, 'testbase')
        self.args_noprint = argparse.Namespace(
            available=False, version=False, bin=None, binary=False, channel=None,
            clean=False, cloud=False, component=False, device='*', gdb=None,
            gdbs=None, gdbs_port=2345, group=False, iface=None, kill_putty=False, list=True,
            listsuites=False, log='./log', my_duts=None, nobuf=None,
            pause_when_external_dut=False,
            putty=False, reset=False, silent=True, skip_case=False,
            skip_rampdown=False, skip_rampup=False,
            status=False, suite=False, tc='all', tc_cfg=None, tcdir=self.testdir,
            testtype=False, type=None, platform_filter=None,
            subtype=None, use_sniffer=False, valgrind=False, valgrind_tool=None,
            verbose=False, repeat=0, feature=None, json=False)
        self.args_tc_no_exist = argparse.Namespace(
            available=False, version=False, bin=None, binary=False, channel=None,
            clean=False, cloud=False, component=False, device='*', gdb=None,
            gdbs=None, gdbs_port=2345, group=False, iface=None, kill_putty=False, list=True,
            listsuites=False, log='./log', my_duts=None, nobuf=None,
            pause_when_external_dut=False, platform_filter=None,
            putty=False, reset=False, silent=True, skip_case=False,
            skip_rampdown=False, skip_rampup=False,
            status=False, suite=False, tc='does_not_exist', tc_cfg=None, tcdir=self.testdir,
            testtype=False, type=None, subtype=None, use_sniffer=False,
            valgrind=False, valgrind_tool=None, verbose=False, repeat=0, feature=None,
            json=False)
        self.args_suite = argparse.Namespace(
            available=False, version=False, bin=None, binary=False, channel=None,
            clean=False, cloud=False, component=False, device='*', gdb=None,
            gdbs=None, gdbs_port=2345, group=False, iface=None,
            kill_putty=False, list=False,
            listsuites=False, log='./log', my_duts=None, nobuf=None,
            pause_when_external_dut=False, platform_filter=None,
            putty=False, reset=False, silent=True, skip_case=False,
            skip_rampdown=False, skip_rampup=False,
            status=False, suite="dummy_suite.json", tc=None, tc_cfg=None,
            tcdir="examples", testtype=False, type=None,
            subtype=None, use_sniffer=False, valgrind=False, valgrind_tool=None,
            verbose=False, repeat=2, feature=None, suitedir="./test/suites", json=False)
        self.args_tc = argparse.Namespace(
            available=False, version=False, bin=None, binary=False, channel=None,
            clean=False, cloud=False, component=False, device='*', gdb=None,
            gdbs=None, gdbs_port=2345, group=False, iface=None, kill_putty=False, list=False,
            listsuites=False, log='./log', my_duts=None, nobuf=None, pause_when_external_dut=False,
            putty=False, reset=False, silent=True, skip_case=False,
            skip_rampdown=False, skip_rampup=False, platform_filter=None,
            status=False, suite=None, tc="test_cmdline", tc_cfg=None,
            tcdir="examples", testtype=False, type="process",
            subtype=None, use_sniffer=False, valgrind=False,
            valgrind_tool=None, verbose=False, repeat=0, feature=None,
            suitedir="./test/suites", forceflash_once=True, forceflash=False,
            stop_on_failure=False, json=False)

    def test_create_suite_suitefile_some_not_found(self):
        suit = TestSuite(args=self.args_suite)
        self.assertEqual(len(suit), 5)

    def test_create_suite_list_success(self):
        suit = TestSuite(args=self.args_noprint)
        self.assertEqual(len(suit), 7, "Suite length ({}) did not match expected length of {}!".format(len(suit), 7))
        self.assertEqual(suit.status, TestStatus.READY)

    def test_create_suite_suitefile_fail(self):
        self.args_suite.suite = "malformed_suite.json"
        with self.assertRaises(SuiteException):
            suite = TestSuite(args=self.args_suite)

    def test_create_suite_list_empty(self):
        suit = TestSuite(args=self.args_tc_no_exist)
        self.assertEqual(len(suit), 1)

    def test_prepare_suite_success(self):
        self.args_suite.suite = "working_suite.json"
        suit = TestSuite(args=self.args_suite)
        self.assertEqual(suit.status, TestStatus.READY)
        tc = suit.get_testcases().get(0)
        self.assertEqual(tc.status, TestStatus.READY)

    def test_prepare_suite_merge_configs(self):
        self.args_suite.suite = "working_suite.json"
        suit = TestSuite(args=self.args_suite)
        tcs = suit.get_testcases()
        self.assertEqual(len(tcs), 2)
        tc = tcs.get(1)
        sconf = tc.get_suiteconfig()
        with open("test/suites/working_suite.json") as f:
            suite = json.load(f)
            cases = suite.get("testcases")
            case2 = cases[1]
            self.assertDictEqual(case2.get("config"), sconf)
        conf = tc.get_final_config()
        self.assertDictEqual(merge(tc.get_instance().get_config(), sconf), conf)

    def test_prepare_suite_merge_configs_missing_tc(self):
        self.args_suite.suite = "suite_missing_one.json"
        suit = TestSuite(args=self.args_suite)
        tcs = suit.get_testcases()
        self.assertEqual(len(tcs), 3)
        tc = tcs.get(2)
        sconf = tc.get_suiteconfig()
        with open("test/suites/suite_missing_one.json") as f:
            suite = json.load(f)
            cases = suite.get("testcases")
            case2 = cases[2]
            self.assertDictEqual(case2.get("config"), sconf)
        conf = tc.get_final_config()
        self.assertDictEqual(merge(tc.get_instance().get_config(), sconf), conf)
        self.assertTrue(isinstance(tcs.get(1), DummyContainer))
        all_duts = conf.get("requirements").get("duts").get("*")
        self.assertIsNotNone(all_duts)
        self.assertIsNone(all_duts.get("should_not"))
        self.assertTrue(all_duts.get("should_be") == "here")

    @mock.patch("icetea_lib.TestSuite.TestSuite.TestSuite._prepare_testcase")
    def test_prepare_suite_fail(self, mock_prep):
        self.args_suite.suite = "working_suite.json"
        mock_prep.side_effect = [TypeError, SyntaxError]
        with self.assertRaises(SuiteException):
            suit = TestSuite(args=self.args_suite)
        with self.assertRaises(SyntaxError):
            suit = TestSuite(args=self.args_suite)

    def test_get_suite_tcs_success(self):
        self.args_suite.suite = "working_suite.json"
        suit = TestSuite(args=self.args_suite)
        self.assertIsNone(suit._get_suite_tcs("dir", []))
        tcs = suit._get_suite_tcs("./examples", 'all')
        self.assertNotEqual(len(tcs), 0)
        # Make sure that the same testcase is found twice and that they both have different instances.
        tcs = suit._get_suite_tcs("./examples", ["sample_process_multidut_testcase", "sample_process_multidut_testcase"])
        self.assertEqual(len(tcs), 2)

    @mock.patch("icetea_lib.TestSuite.TestSuite.TestcaseFilter")
    def test_get_suite_tcs_errors(self, mock_filter):
        self.args_suite.suite = "working_suite.json"
        mock_filter.side_effect = [TypeError, {}]
        with self.assertRaises(SuiteException):
            suit = TestSuite(args=self.args_suite)

    def test_print_list_testcases(self):
        suit = TestSuite(args=self.args_noprint)
        table = suit.list_testcases()

    def test_get_suite_files(self):
        lst = TestSuite.get_suite_files("./test/suites")
        self.assertEqual(len(lst), 5)

    def test_load_suite_list(self):

        self.args_tc.tc = "tc_no_exist"
        suit = TestSuite(args=self.args_tc)
        suit._load_suite_list()
        self.assertEqual(len(suit), 1)
        suit.args.tcdir = "./test/suites"
        suit._load_suite_list()
        self.assertEqual(len(suit), 1)
        suit.args.status = 2
        self.assertFalse(suit._load_suite_list())
        suit.args.tc = False
        suit.args.status = "released"
        suit.args.tcdir = "examples"
        self.assertIsNot(suit._load_suite_list(), False)

    @mock.patch("icetea_lib.TestSuite.TestSuite.TestSuite._create_tc_list")
    def test_run(self, mock_tclist):
        testsuite = TestSuite(args=self.args_tc)
        cont1 = mock.MagicMock()
        pass_result = Result()
        pass_result.set_verdict('pass', 0, 10)
        fail_result = Result()
        fail_result.set_verdict('fail', 1000, 10)
        skipped_result = Result()
        skipped_result.set_verdict('skip', 0, 1)
        resultlist = ResultList()
        resultlist.append(pass_result)
        testsuite._default_configs["retryCount"] = 1
        cont1.run.side_effect = [pass_result, fail_result, skipped_result, KeyboardInterrupt, fail_result, pass_result]
        cont_reslist = mock.MagicMock()
        cont_reslist.run = mock.MagicMock()
        cont_reslist.run.return_value = resultlist
        # Passing result
        testsuite._testcases = []
        testsuite._testcases.append(cont1)
        testsuite._results = []
        testsuite.run()
        self.assertEqual(testsuite.status, TestStatus.FINISHED)
        self.assertEqual(len(testsuite._results), 1)
        self.assertEqual(testsuite._results[0].get_verdict(), "pass")
        self.assertTrue(self.args_tc.forceflash)

        # ResultList as result
        testsuite._testcases = []
        testsuite._testcases.append(cont_reslist)
        testsuite._results = []
        testsuite.run()
        self.assertEqual(testsuite.status, TestStatus.FINISHED)
        self.assertEqual(len(testsuite._results), 1)
        self.assertEqual(testsuite._results[0].get_verdict(), "pass")

        # Failing result, no retry
        testsuite._testcases = []
        testsuite._testcases.append(cont1)
        testsuite._results = []
        testsuite.run()
        self.assertEqual(testsuite.status, TestStatus.FINISHED)
        self.assertEqual(len(testsuite._results), 1)
        self.assertEqual(testsuite._results[0].get_verdict(), "fail")

        # skipped result
        testsuite._testcases = []
        testsuite._testcases.append(cont1)
        testsuite._results = []
        testsuite.run()
        self.assertEqual(testsuite.status, TestStatus.FINISHED)
        self.assertEqual(len(testsuite._results), 1)
        self.assertEqual(testsuite._results[0].get_verdict(), "skip")

        # Interrupt
        cont2 = mock.MagicMock()
        cont2.run = mock.MagicMock()
        testsuite._testcases = []
        testsuite._testcases.append(cont1)
        testsuite._testcases.append(cont2)
        testsuite._results = []
        testsuite.run()
        self.assertEqual(testsuite.status, TestStatus.FINISHED)
        self.assertEqual(len(testsuite._results), 0)
        cont2.run.assert_not_called()

        # Failing result, retried
        testsuite._testcases = []
        testsuite._testcases.append(cont1)
        testsuite._results = []
        testsuite._default_configs["retryReason"] = "includeFailures"
        testsuite.run()
        self.assertEqual(testsuite.status, TestStatus.FINISHED)
        self.assertEqual(len(testsuite._results), 2)
        self.assertEqual(testsuite._results[0].get_verdict(), "fail")
        self.assertEqual(testsuite._results[1].get_verdict(), "pass")

        self.args_tc.repeat = 2
        testsuite._testcases = []
        testsuite._testcases.append(cont1)
        testsuite._results = []
        cont1.run.side_effect = [pass_result, pass_result, pass_result, pass_result]
        testsuite.run()
        self.assertEqual(testsuite.status, TestStatus.FINISHED)
        self.assertEqual(len(testsuite._results), 2)
        self.assertEqual(testsuite._results[0].get_verdict(), "pass")
        self.assertFalse(self.args_tc.forceflash)

        # Failing result, stop_on_failure
        self.args_tc.stop_on_failure = True
        self.args_tc.repeat = 1
        testsuite._default_configs["retryCount"] = 0
        testsuite._testcases = []
        testsuite._testcases.append(cont1)
        testsuite._results = []
        cont1.run.side_effect = [pass_result]
        cont2 = mock.MagicMock()
        cont2.run = mock.MagicMock()
        cont2.run.side_effect = [fail_result]
        cont3 = mock.MagicMock()
        cont3.run = mock.MagicMock()
        cont3.run.side_effect = [pass_result]
        testsuite._testcases.append(cont2)
        testsuite.run()
        self.assertEqual(testsuite.status, TestStatus.FINISHED)
        self.assertEqual(len(testsuite._results), 2)
        self.assertEqual(testsuite._results[0].get_verdict(), "pass")
        self.assertEqual(testsuite._results[1].get_verdict(), "fail")

        # Skipped result, stop_on_failure
        self.args_tc.stop_on_failure = True
        self.args_tc.repeat = 0
        testsuite._testcases = []
        testsuite._testcases.append(cont1)
        testsuite._results = []
        cont1.run.side_effect = [skipped_result]
        cont2 = mock.MagicMock()
        cont2.run = mock.MagicMock()
        cont2.run.side_effect = [pass_result]
        testsuite._testcases.append(cont2)
        testsuite.run()
        self.assertEqual(testsuite.status, TestStatus.FINISHED)
        self.assertEqual(len(testsuite._results), 2)
        self.assertEqual(testsuite._results[0].get_verdict(), "skip")
        self.assertEqual(testsuite._results[1].get_verdict(), "pass")

    @mock.patch("icetea_lib.TestSuite.TestSuite.TestSuite._create_tc_list")
    @mock.patch("icetea_lib.TestSuite.TestSuite.os.path")
    def test_load_suite_file(self, mock_path, mock_tc):
        mock_path.join = mock.MagicMock()
        mock_path.join.return_value = "path"
        mock_path.exists = mock.MagicMock()
        mock_path.exists.return_value = False
        suit = TestSuite(args=self.args_suite)
        self.assertIsNone(suit._load_suite_file(1, "dir"))
        self.assertIsNone(suit._load_suite_file("name", "dir"))
        with mock.patch.object(suit, "cloud_module") as mock_cm:
            mock_cm.get_suite = mock.MagicMock()
            mock_cm.get_suite.return_value = None
            self.assertIsNone(suit._load_suite_file("name", "dir"))



if __name__ == '__main__':
    unittest.main()



# pylint: disable=missing-docstring,unused-variable

"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import unittest
from random import randint
from threading import Event as EventFlag
from time import sleep

import mock

from icetea_lib.Events.EventMatcher import EventMatcher
from icetea_lib.Events.Generics import EventTypes, Event, Observer
from icetea_lib.tools.tools import IS_PYTHON3


class MockLineProvider(object):  # pylint: disable=too-few-public-methods
    def __init__(self):
        self.counter = 0

    def readline(self, timeout):
        if self.counter == 2:
            self.counter = 0
            return "found"
        sleep(randint(0, timeout))
        self.counter += 1
        return "nothing"


class EventTestcase(unittest.TestCase):

    def test_resolve_match_data(self):
        test_object = mock.MagicMock()
        callback = mock.MagicMock()
        event_flag = EventFlag()
        event_matcher = EventMatcher(EventTypes.DUT_LINE_RECEIVED, "test", test_object,
                                     flag=event_flag, callback=callback)
        event = Event(EventTypes.DUT_LINE_RECEIVED, test_object, "test")
        callback.assert_called_once_with(test_object, "test")
        self.assertTrue(event_flag.isSet())
        event_flag.clear()
        callback.reset_mock()
        # Recreate matcher because it forgets itself once it has matched once.
        event_matcher = EventMatcher(EventTypes.DUT_LINE_RECEIVED, "regex:test*", test_object,
                                     flag=event_flag, callback=callback)
        event = Event(EventTypes.DUT_LINE_RECEIVED, test_object, "nothing")
        self.assertFalse(event_flag.isSet())
        event = Event(EventTypes.DUT_LINE_RECEIVED, test_object, "test1")
        callback.assert_called_once_with(test_object, "test1")
        self.assertTrue(event_flag.isSet())
        event_flag.clear()
        callback.reset_mock()
        event_matcher = EventMatcher(EventTypes.DUT_LINE_RECEIVED, "regex:test:[0-9]", test_object,
                                     flag=event_flag, callback=callback)
        event = Event(EventTypes.DUT_LINE_RECEIVED, test_object, "test")
        self.assertFalse(event_flag.isSet())
        event = Event(EventTypes.DUT_LINE_RECEIVED, test_object, "test:1")
        callback.assert_called_once_with(test_object, "test:1")
        self.assertTrue(event_flag.isSet())

    def test_resolve_data_no_caller(self):
        test_object = mock.MagicMock()
        callback = mock.MagicMock()
        event_flag = EventFlag()
        eventmatcher = EventMatcher(EventTypes.DUT_LINE_RECEIVED, "test", caller=None,
                                    flag=event_flag, callback=callback)
        event = Event(EventTypes.DUT_LINE_RECEIVED, test_object, "test")
        callback.assert_called_once_with(test_object, "test")
        self.assertTrue(event_flag.isSet())

    def test_resolve_data_decodefail(self):
        test_object = mock.MagicMock()
        callback = mock.MagicMock()
        event_flag = EventFlag()
        if IS_PYTHON3:
            event_matcher = EventMatcher(EventTypes.DUT_LINE_RECEIVED,
                                         "\x00\x00\x00\x00\x00\x00\x01\xc8", test_object,
                                         flag=event_flag, callback=callback)
        else:
            event_matcher = EventMatcher(EventTypes.DUT_LINE_RECEIVED,
                                         repr("\x00\x00\x00\x00\x00\x00\x01\xc8"), test_object,
                                         flag=event_flag, callback=callback)
        event = Event(EventTypes.DUT_LINE_RECEIVED, test_object, "\x00\x00\x00\x00\x00\x00\x01\xc8")
        callback.assert_called_once_with(test_object, "\x00\x00\x00\x00\x00\x00\x01\xc8")
        self.assertTrue(event_flag.isSet())
        event_flag.clear()

    def test_observer(self):  # pylint: disable=no-self-use
        obs = Observer()
        callback = mock.MagicMock()
        obs.observe(EventTypes.DUT_LINE_RECEIVED, callback)
        callback.assert_not_called()
        event = Event(2, "data")
        callback.assert_not_called()
        event = Event(EventTypes.DUT_LINE_RECEIVED, "data")
        callback.assert_called_once_with("data")


if __name__ == '__main__':
    unittest.main()
